% Funtion converting an OpenSimModel and OpenSimState into a function which 
% can be passed as a input to a Matlab integrator or optimization routine
%
% Input:
%   t: Time at the current step
%   x: Matlab column matrix of state values at the current step
%   controlsFunc: Function handle for a function which computes the control vector
%   osimModel: org.opensim.Modeling.Model object
%   osimState: org.opensim.Modeling.State object
%
% Output:
%   x_dot is a Matlab column matrix of the derivative of the state values
%
function [x_dot, controlValues] = opensim_actuator(t, x, controlsFunc, osimModel, osimState)

% Update state with current values
osimState.setTime(t) ;
numVar = osimModel.getNumStateVariables() ;
for i = 0:1:numVar - 1
    CoordName = osimModel.getStateVariableNames().get(i) ;
    osimModel.setStateVariable(osimState, char(CoordName), x(i + 1))
end

% Update the state velocity calculations
osimModel.computeStateVariableDerivatives(osimState) ;

% Update model with control values
if(~isempty(controlsFunc))
    controlVector = controlsFunc(osimModel, osimState) ;
    osimModel.setControls(osimState, controlVector) ;
    for i = 1:osimModel.getNumControls()
        controlValues(1) = controlVector.get(i - 1) ;
    end
end

% Update the derivative calculations 
stateDerivs = osimModel.computeStateVariableDerivatives(osimState) ;

x_dot = zeros(numVar, 1) ;
% Set output variable to new state
for i = 0:1:numVar - 1
    x_dot(i + 1, 1) = stateDerivs.get(i) ;
end

end