% Function opens model, edits the initial state, runs a forward simulation using built-in Matlab integrators, then saves the Output file.
% === code is based off Main_WalkerForwardSimWithControls.m from the Dynamic Walker package.

function [kinematics, kinetics] = opensim_simulateControls
% Import OpenSim Libraries
import org.opensim.modeling.* ;

% Open Model
osimModel = Model([pwd, '\testModel.osim']) ;

% Use the visualizer (must be done before the call to init system)
osimModel.setUseVisualizer(1) ;

% Initialize the system and get the initial state
osimState = osimModel.initSystem() ;

% Calculate the derivatives
stateDerivVector = osimModel.computeStateVariableDerivatives(osimState) ; %#ok<NASGU>

% Create the Initial State matrix from the OpenSim state
numVar     = osimModel.getNumStateVariables() ;
InitStates = zeros(numVar, 1) ;
for i = 0:1:numVar - 1
    InitStates(i + 1, 1) = osimModel.getStateValues(osimState).get(i) ;
end

% Set the integrator options
timeSpan          = 0:1e-4:0.5 ;
integratorOptions = odeset('AbsTol', 1E-5) ;

% Set up the visualizer
if(osimModel.getUseVisualizer())
    visualizeHandle = @(t, x, flag) OpenSimVisualizeFunction(t, x, flag, osimModel, osimState) ;
    integratorOptions.OutputFcn = visualizeHandle ;
end

% Function handles
global v ; v = 1 ;
controlsFunc = @opensim_actControl ; % handle to control function
actuatorFunc = @(t,x) opensim_actuator(t, x, controlsFunc, osimModel, osimState) ; % handle to link OpenSim and Matlab

% Integrate the system equations
tic ;
integratorFunc    = str2func('ode45') ;
[Time, Variables] = integratorFunc(actuatorFunc, timeSpan, InitStates, integratorOptions) ;
toc()

% Output Data
kinematics           = struct() ;
kinematics.name      = [char(osimModel.getName()), '_states'] ;
kinematics.nRows     = size(Time, 1) ;
kinematics.nColumns  = size(Time, 2) + size(Variables, 2) ;
kinematics.labels    = cell(1, kinematics.nColumns) ;
kinematics.labels{1} = 'time' ;
for j = 2:kinematics.nColumns
    kinematics.labels{j} = char(osimModel.getStateVariableNames().getitem(j - 2)) ;
end
kinematics.data = [Time, Variables] ;

global valSave
kinetics = valSave ; clearvars -global valSave v
end

% Visualizer function
function status = OpenSimVisualizeFunction(t, x, flag, openSimModel, openSimState)
status = 0 ;

% Flag has three values: init (initialization), empty (succesful step made) and done (cleanup)
if(isempty(flag))
    openSimState.setTime(t(1)) ;
    numVar = openSimModel.getNumStateVariables() ;
    for i = 0:1:numVar - 1
        CoordName = openSimModel.getStateVariableNames().get(i);
        openSimModel.setStateVariable(openSimState, char(CoordName), x(i + 1));
    end
    openSimModel.getVisualizer().show(openSimState) ;
end
end